#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys, os, string, re, random
from bottle import route, run, request, HTTPError, HTTPResponse, ServerAdapter
import glob
import json
import time
import traceback
import ConfigParser
import logging
import logging.config
from datetime import datetime

restServiceHost = '0.0.0.0'
restServicePort = 8081
cardTransformRules = []
servicePath = os.path.dirname(os.path.abspath(__file__)) + '/'
callCount = {}
testMode = False
ruleList = {}
verificationCode = ''
message = u''
serverJavaExceptions = ['UnsupportedVerificationTypeException', 'WrongAccountingClusterException', 'GeneralVerifierException', 'InvalidKeyException', 'CardNotFoundException', 'AdditionalCardSearchException', 'NotImplementedException']

try:
    logging.config.fileConfig(servicePath + 'conf/logging.properties')
except Exception, e:
    print 'Ошибка `%s`' % e
    logging.basicConfig(format = u'%(levelname)-8s [%(asctime)s] %(message)s', level = logging.DEBUG, filename = u'cardverification.log')

def getRule(funcName):
    rule = None
    func = funcName.lower()
    if testMode and func in ruleList:
        if func not in callCount:
            callCount.update({func: 0})
        callCount[func] += 1

        curNumber = callCount[func]
        if curNumber >= len(ruleList[func]) or ruleList[func][curNumber] == None: 
            curNumber = 0
        rule = ruleList[func][curNumber]
    return rule

def getResult(status,comment):
    return {u'status': status, u'comment': comment}

def checkBaseRule(rule):
    if rule == 'errorConnection':
        logging.error(u'Тест.Ошибка соединения')
        raise HTTPError(404)
    elif rule in serverJavaExceptions:
        logging.error(u'Тест.Ошибка сервера')
        resp = HTTPResponse(body='{"errorCode": 500, "url": "/CSrest/rest/ident/cards", "message": "another server error", "errorClassName": "%s"}' % rule, status=400)
        resp.set_header('content_type', 'application/json')
        raise resp
    elif rule == 'internalServerError':
        logging.warning(u'Тест.Внутренняя ошибка сервера')
        return 1
    elif rule == 'externalServerError':
        logging.warning(u'Тест.Ошибка во внешнем шлюзе')
        return 2
    return None

def checkRequiredParameters():
    logging.info(u'Проверка обязательных параметров')
    if request.query.get('idcardgroup') is None or request.query.get('number') is None:
        logging.error(u'Отсутсвуют обязательные параметры')
        raise HTTPError(422) # отсутствуют обязательные параметры в запросе
    logging.info(u'Проверка окончена')

@route('/CSrest/rest/ident/cards', method='GET')
def vefiryCard():
    global cardTransformRules
    global verificationCode
    global message
    checkRequiredParameters()
    cardNumber = request.query.get('number')
    verificationType = request.query.get('vertype')

    rule = getRule('verifyCard')
    verificationResult = checkBaseRule(rule)
    if verificationResult is None:
        verificationResult = 0

    if verificationType is None or verificationType.upper() == 'AUTO':
        verificationcode = ''
    elif not verificationCode:
        verificationcode = ''.join(random.choice(string.ascii_lowercase + string.ascii_uppercase + string.digits) for _ in range(6))
    logging.info(u'Код верификации - \'%s\'' % verificationcode)

    for pattern in cardTransformRules:
        if not re.search(pattern[0], cardNumber):
            continue
        cardNumber = re.sub(pattern[0], pattern[1], request.query.get('number'))
        break
    logging.info(u'Номер карты - \'%s\'' % cardNumber)

    result = {'cardNumber': cardNumber, 'verificationCode': verificationcode, 'verificationResult': verificationResult,
              'message': message}
    return json.dumps(result, ensure_ascii=False)

def init():
    logging.info(u'Чтение настроек')
    global restServiceHost
    global restServicePort
    global testMode
    global verificationCode
    global message

    conf = ConfigParser.RawConfigParser()
    conf.read(servicePath + 'conf/config.ini')
    restServiceHost = conf.get('main', 'host')
    restServicePort = conf.getint('main', 'port')
    testMode = conf.getboolean('main', 'testMode')
    verificationCode = conf.get('main', 'verificationCode')
    message = conf.get('main', 'message')

    for i in range(0, 10):
        if conf.has_option('main', 'mask%s' % i) and conf.has_option('main', 'rule%s' % i):
            cardTransformRules.append((conf.get('main', 'mask%s' % i), conf.get('main', 'rule%s' % i)))

    if testMode:
        for key, val in conf.items('test'):
            rule = key.split('.')
            beg = 0
            end = 0
            func = rule[0]
            if not func in ruleList:
                ruleList[func] =[] 
            if len(rule) > 1:
                interval = rule[1].split('-')
                beg = int(interval[0])
                end = int(interval[0])
                if len(interval) > 1:
                    end = int(interval[1])
            i = beg
            while i <= end:
                while i >= len(ruleList[func]):
                    ruleList[func].append(None)
                ruleList[func][i] = val
                i +=1
        logging.info(u'Заданы правила для сервиса: %s' % ruleList)

def start():
    logging.info(u'Запускаем сервис')
    try:
        init()
        logging.info(u'Стартуем на %s:%s' % (restServiceHost, restServicePort))
        run(host=restServiceHost, port=restServicePort)
    except Exception as exception:
        print(exception)

if __name__ == '__main__':
    start()
